import { Injectable, OnDestroy } from "@angular/core";
import { Observable, Subject, timer } from "rxjs";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import { HttpClient } from "@angular/common/http";
import { retry, switchMap, takeUntil } from "rxjs/operators";

/**
 * Service that implements a polling mechanism for data that is retrieved using HTTP GET requests
 */
@Injectable({
  providedIn: "root",
})
export class PollingDataUpdaterService implements OnDestroy {
  private stopPolling = new Subject();

  constructor(private httpClient: HttpClient) {}

  /**
   * Get data poller observer that makes periodic GET requests; allows for periodic real-time updates to UI components
   * that get results from REST
   * @param url  URL to make periodic GET requests to
   * @return Observable<JsonObject>  Data poller observable
   */
  public getDataPoller$(url: string): Observable<JsonObject> {
    return timer(1, 3000).pipe(
      switchMap(() => this.httpClient.get<JsonObject>(url)),
      retry(),
      takeUntil(this.stopPolling),
    );
  }

  ngOnDestroy(): void {
    this.stopPolling.next();
    this.stopPolling.complete();
  }
}
